sidv_SharedIDVote

	//===========================================================================================
	var
		list
			sidv_voters
			sidv_voter2vote

	//===========================================================================================
	proc
		//---------------------------------------------------------------------------------------
		//	Returns the ID of Voter.  You'll want to overwrite this if you use different IDs.
		//	By default:
		//		If Voter voted, then the ID it used is returned.
		//		Next it tries to return the IP address of Voter if Voter is or currently has a client.
		//		It returns null otherwise, which is still a valid ID.
		sidv_ID(Voter)
			if (Voter in src.sidv_voter2vote)	return src.sidv_voter2vote[Voter]

			var/client/cVoter
			if ( istype(Voter,/client) )
				cVoter	= Voter
				return cVoter.address

			if ( ismob(Voter) )
				var/mob/mVoter	= Voter
				cVoter			= mVoter.client
				if (cVoter)		return cVoter.address

			return null

		//---------------------------------------------------------------------------------------
		//	Adds Voter as a legal voter and returns TRUE on success.
		sidv_AddVoter(Voter)
			if ( src.sidv_IsVoter(Voter) )	return TRUE
			if (!src.sidv_voters)			src.sidv_voters	= list()
			src.sidv_voters					+= Voter
			return TRUE

		//---------------------------------------------------------------------------------------
		//	Removes Voter's vote (if voted) and returns TRUE on success.
		sidv_CancelVote(Voter)
			if ( src.sidv_DidVote(Voter) )
				src.sidv_voter2vote	-= Voter
				return TRUE
			return FALSE

		//---------------------------------------------------------------------------------------
		//	Returns wether or not Voter can vote.
		//	Voters must be registered.
		//	They must not have voted yet.
		//	They must be the first person with their ID to have voted.
		sidv_CanVote(Voter)
			if ( !src.sidv_IsVoter(Voter) )					return FALSE
			if ( src.sidv_DidVote(Voter) )					return FALSE
			if ( src.sidv_IDDidVote(src.sidv_ID(Voter)) )	return FALSE
			return TRUE

		//---------------------------------------------------------------------------------------
		//	Removes all votes.
		sidv_ClearVotes()
			src.sidv_voter2vote	= null
			return TRUE

		//---------------------------------------------------------------------------------------
		//	Removes all voters and all votes.
		sidv_ClearVoters()
			src.sidv_voters	= null
			return src.sidv_ClearVotes()

		//---------------------------------------------------------------------------------------
		//	Returns whether or not Voter voted.
		sidv_DidVote(Voter)
			return (Voter in src.sidv_voter2vote)

		//---------------------------------------------------------------------------------------
		//	Returns whether or not someone with ID voted.
		sidv_IDDidVote(ID)
			return ( ID in src.sidv_IDsVoted() )

		//---------------------------------------------------------------------------------------
		//	Returns a list of the IDs of all of the voters.
		//	Each ID is only listed once.
		sidv_IDsVoters()
			.					= list()
			var
				voter
				id
			for(voter in src.sidv_voters)
				id				= src.sidv_ID(voter)
				if (id in .)	continue
				.				+= id

		//---------------------------------------------------------------------------------------
		//	Returns a list of the IDs which voted.
		sidv_IDsVoted()
			.										= list()
			for(var/voter in src.sidv_voter2vote)	.	+= src.sidv_voter2vote[voter]

		//---------------------------------------------------------------------------------------
		//	Returns whether or not Candidate has been added as a voter.
		sidv_IsVoter(Candidate)
			return (Candidate in src.sidv_voters)

		//---------------------------------------------------------------------------------------
		//	Makes Voter unable to vote.  If Voter voted, then the vote is removed.
		//	Returns TRUE on success.
		//	Note that this could cause the vote to pass.
		sidv_RemoveVoter(Voter)
			if (Voter in src.sidv_voters)
				src.sidv_CancelVote(Voter)
				src.sidv_voters	-= Voter
				if ( length(src.sidv_voter2vote) >= src.sidv_RequiredVotes() )
					return src.sidv_VotePassed()
				return TRUE
			return FALSE

		//---------------------------------------------------------------------------------------
		//	Returns the number of required votes.
		//	By default, this is the ceiling of half of the unique voter ID.
		//	If players are even, then another vote is added.
		//	This means that both players will have to vote in a two player game.
		sidv_RequiredVotes()
			var/numVoters		= length( src.sidv_IDsVoters() )
			.					= -round(-numVoters/2)
			//	Odd?
			if (numVoters%2)	return .
			//	Even.
			.					+= 1

		//---------------------------------------------------------------------------------------
		//	Voter votes for the notion to succeed.
		//	Returns FALSE if Voter can not currently vote.
		//	If enough votes are met, then the result of the passing vote is returned.
		//	Otherwise, returns TRUE.
		sidv_Vote(Voter)
			if ( !src.sidv_CanVote(Voter) )	return FALSE

			if (!src.sidv_voter2vote)	src.sidv_voter2vote	= list()
			src.sidv_voter2vote[Voter]	= src.sidv_ID(Voter)

			if ( length(src.sidv_voter2vote) >= src.sidv_RequiredVotes() )
				return src.sidv_VotePassed()

			return TRUE

		//---------------------------------------------------------------------------------------
		//	The vote has passed.  You'll want to overwrite this to define what happened.
		//	By default, the votes are cleared.
		//	Returns TRUE.
		sidv_VotePassed()
			src.sidv_ClearVotes()
			return TRUE

		//---------------------------------------------------------------------------------------
		//	Returns the list of current voters.
		sidv_Voters()
			if (src.sidv_voters)	return src.sidv_voters.Copy()
			return list()
